#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include "../../lib/backend.h"
#include "../../lib/config.h"
#include "../../lib/game_logic.h"

void msg(const char* msg) {
  printf("\n%s\n", msg);
}

bool choice_msg(const char* msg) {
  printf("\n%s\n", msg);
  printf("0) No\t1) Yes\nEnter choice: ");
  uint8_t choice;
  scanf("%hhu", &choice);
  return (choice == 1) ? true : false;
}

void render_ui(const game_state* state) {
  printf("\033[H\033[J");
  printf("Mines: %u\tFlags: %u\n", state->mines, state->flags);
}

void render_map(const game_state* state) {
  for (uint32_t y = 0; y < state->height; ++y) {
    if (y == 0) {
      printf("%-2c ", ' ');
      for (uint32_t x = 0; x < state->width; ++x) {
        printf("%-2d ", x);
      }
      printf("\n");
    }
    printf("%-2d ", y);
    for (uint32_t x = 0; x < state->width; ++x) {
      const uint32_t index = y * state->width + x;
      const field* cell = &state->map[index];
            
      if (cell->is_protected) {
        printf("%-2c ", 'F');
      } else {
        switch (cell->state) {
        case OPENED:
          if (cell->near_bombs > 0) printf("%-2d ", cell->near_bombs);
          else printf("%-2c ", '.');
          break;
        case DESTROYED:
          printf("%-2c ", DESTROYED);
          break;
        case CLOSED:
        case MINED:
          printf("%-2c ", CLOSED);
          break;
        default:
          fprintf(stderr, "Unknown state: %c", cell->state);
          printf("%-2c ", '?');
        }
      }
    }
    printf("\n");
  }
}

void action(game_state* state) {
  uint32_t x;
  printf("Enter x: ");
  scanf("%d", &x);

  uint32_t y;
  printf("Enter y: ");
  scanf("%d", &y);

  printf("0) Open\t1) Set flag\nEnter operation: ");
  uint8_t choice;
  scanf("%hhu", &choice);

  if (choice == 1) {
    make_action(state, x, y, SET_FLAG);
  } else {
    make_action(state, x, y, OPEN);
  }
}

int game_loop() {
  config settings;
  bool result = read_config(&settings);
  if (!result) {
    msg("CONFIG NOT LOADED\n");
    return 1;
  }
  bool restart = false;
  do {
    game_state world;
    game_state_init(&world, &settings);
    calculate_near_bombs(&world);

    
    while (world.status == RUNNING) {
      render_ui(&world);
      render_map(&world);
      action(&world);
      calculate_near_bombs(&world);
    }
    game_state_free(&world);

    printf("\033[H\033[J");
    switch (world.status) {
    case LOSE:
      msg("IT'S OVER");
      break;
    case WIN:
      msg("YOU WIN");
      break;
    default:
      msg("YOU HACKER?");
    }
    restart = choice_msg("Restart?");
  } while (restart);

  return 0;
}

