#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include "lib/config.h"

#define BUFFER_LEN 32

options get_option(const char *str) {
    if (strcmp(str, "width") == 0) return WIDTH;
    if (strcmp(str, "height") == 0) return HEIGHT;
    if (strcmp(str, "mines") == 0) return MINES;
    if (strcmp(str, "flags") == 0) return FLAGS;
    return UNKNOWN;
}

void strip(char** str) {
  char *start = *str;
 while (isspace(*start)) start++;
  char *end = start + strlen(start) - 1;
  while (end > start && isspace(*end)) end--;
  end[1] = '\0';
  *str = start;
}

bool read_config(config *conf) {
  FILE* file = fopen(DEFAULT_CONFIG_FILENAME, "rb");
  if (!file) {
    fprintf(stderr, "%s: %s\n", DEFAULT_CONFIG_FILENAME, strerror(errno));
    return false;
  }

  char buf[BUFFER_LEN + 1] = {0};
  while (fgets(buf, BUFFER_LEN, file) != NULL) {
    if (strchr(buf, '=') == NULL) continue;
    char* ptr_save;
    char* key = strtok_r(buf, "=", &ptr_save);
    char* value = strtok_r(NULL, "=", &ptr_save);
    if (key == NULL || value == NULL) {
      fprintf(stderr, "Error when parsing the config\n");
      fclose(file);
      return false;
    }
    strip(&key);
    strip(&value);
    switch (get_option(key)) {
    case WIDTH:
      conf->width = atoi(value);
      break;
    case HEIGHT:
      conf->height = atoi(value);
      break;
    case MINES:
      conf->mines = atoi(value);
      break;
    case FLAGS:
      conf->flags = atoi(value);
      break;
    default:
      fprintf(stderr, "There is no such option (%s), please use: width, height, mines, flags\n", key);
      fclose(file);
      return false;
    }
  }

  if (conf->width == 0 || conf->height == 0 || conf->mines == 0 || conf->flags == 0) {
    fprintf(stderr, "Error: missing required parameters\n");
    return false;
  }
  if (conf->width > WIDTH_LIMIT) {
    fprintf(stderr, "The width (%u) exceeds the limit (%u)\n", conf->width, WIDTH_LIMIT);
    fclose(file);
    return false;
  }
  if (conf->height > HEIGHT_LIMIT) {
    fprintf(stderr, "The height (%u) exceeds the limit (%u)\n", conf->height, HEIGHT_LIMIT);
    fclose(file);
    return false;
  }
  if (conf->flags > FLAGS_LIMIT) {
    fprintf(stderr, "The flags (%u) exceeds the limit (%u)\n", conf->flags, FLAGS_LIMIT);
    fclose(file);
    return false;
  }
  if (conf->mines > MINES_LIMIT) {
    fprintf(stderr, "The mines (%u) exceeds the limit (%u)\n", conf->mines, MINES_LIMIT);
    fclose(file);
    return false;
  }
  if (conf->mines > conf->width * conf->height) {
    fprintf(stderr, "The number of mines exceeds the number of cells\n");
    fclose(file);
    return false;
  }

  fclose(file);
  return true;
}
